"""
Xpert client
"""

import json

import requests

from django.conf import settings

CONNECT_TIMOUET_SECONDS = 5
READ_TIMEOUT_SECONDS = 20


def chat_completion(prompt, role):
    """
    Generate response using xpert api.

    Arguments:
        prompt (str): ChatGPT prompt
        role (str): ChatGPT role to assume for the prompt.

    Returns:
        (str): Prompt response from OpenAI.
    """
    headers = {
        'Content-Type': 'application/json',
    }

    body = {
        'messages': [{'role': role, 'content': prompt},],
        'client_id': settings.ENTERPRISE_ANALYSIS_CLIENT_ID,
        'system_message': settings.ENTERPRISE_ANALYSIS_SYSTEM_PROMPT
    }

    response = requests.post(
        settings.CHAT_COMPLETION_API_V2,
        headers=headers,
        data=json.dumps(body),
        timeout=(CONNECT_TIMOUET_SECONDS, READ_TIMEOUT_SECONDS)
    )

    return response.json().get('content')



# Generated by Django 2.2.14 on 2020-07-08 16:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0101_move_data_to_saved_for_later'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomer',
            name='enable_portal_saml_configuration_screen',
            field=models.BooleanField(default=False, help_text='Specifies whether to allow access to the saml configuration screen in the admin portal'),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomer',
            name='enable_portal_saml_configuration_screen',
            field=models.BooleanField(default=False, help_text='Specifies whether to allow access to the saml configuration screen in the admin portal'),
        ),
    ]



"""
Views for enterprise customer invite keys.
"""

from django_filters.rest_framework import DjangoFilterBackend
from edx_rbac.decorators import permission_required
from edx_rest_framework_extensions.auth.jwt.authentication import JwtAuthentication
from rest_framework import filters, permissions, status
from rest_framework.authentication import SessionAuthentication
from rest_framework.decorators import action
from rest_framework.response import Response
from rest_framework.status import HTTP_200_OK, HTTP_201_CREATED, HTTP_422_UNPROCESSABLE_ENTITY

from django.shortcuts import get_object_or_404

from enterprise import models
from enterprise.api.filters import EnterpriseCustomerInviteKeyFilterBackend
from enterprise.api.utils import get_ent_cust_from_enterprise_customer_key
from enterprise.api.v1 import serializers
from enterprise.api.v1.views.base_views import EnterpriseReadWriteModelViewSet
from enterprise.errors import LinkUserToEnterpriseError
from enterprise.logging import getEnterpriseLogger
from enterprise.utils import track_enterprise_user_linked

LOGGER = getEnterpriseLogger(__name__)


class EnterpriseCustomerInviteKeyViewSet(EnterpriseReadWriteModelViewSet):
    """
    API for accessing enterprise customer keys.
    """
    queryset = models.EnterpriseCustomerInviteKey.objects.all()
    authentication_classes = (JwtAuthentication, SessionAuthentication)
    permission_classes = (permissions.IsAuthenticated,)

    filter_backends = (filters.OrderingFilter, DjangoFilterBackend, EnterpriseCustomerInviteKeyFilterBackend)
    http_method_names = ['get', 'post', 'patch']

    def get_serializer_class(self):
        """
        Use a special serializer for any requests that aren't read-only.
        """
        if self.request.method in ('POST', 'DELETE'):
            return serializers.EnterpriseCustomerInviteKeyWriteSerializer

        if self.request.method == 'PATCH':
            return serializers.EnterpriseCustomerInviteKeyPartialUpdateSerializer

        return serializers.EnterpriseCustomerInviteKeyReadOnlySerializer

    def retrieve(self, request, *args, **kwargs):
        invite_key = get_object_or_404(models.EnterpriseCustomerInviteKey, pk=kwargs['pk'])
        serializer = self.get_serializer(invite_key)
        return Response(serializer.data)

    @permission_required('enterprise.can_access_admin_dashboard')
    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)

    @permission_required('enterprise.can_access_admin_dashboard')
    @action(methods=['get'], detail=False, url_path='basic-list')
    def basic_list(self, request, *args, **kwargs):
        """
        Unpaginated list of all invite keys matching the filters.
        """
        queryset = self.get_queryset()
        queryset = self.filter_queryset(queryset)
        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)

    @permission_required(
        'enterprise.can_access_admin_dashboard',
        fn=lambda request: request.data.get('enterprise_customer_uuid')
    )
    def create(self, request, *args, **kwargs):
        return super().create(request, *args, **kwargs)

    @permission_required(
        'enterprise.can_access_admin_dashboard',
        fn=lambda request, pk: get_ent_cust_from_enterprise_customer_key(pk)
    )
    def partial_update(self, request, *args, **kwargs):
        try:
            return super().partial_update(request, *args, **kwargs)
        except ValueError as ex:
            return Response({'detail': str(ex)}, status=HTTP_422_UNPROCESSABLE_ENTITY)

    @permission_required(
        'enterprise.can_access_admin_dashboard',
        fn=lambda request, pk: get_ent_cust_from_enterprise_customer_key(pk)
    )
    def destroy(self, request, *args, **kwargs):
        return super().destroy(request, *args, **kwargs)

    @action(methods=['post'], detail=True, url_path='link-user')
    def link_user(self, request, pk=None):
        """
        Post
        Links user using enterprise_customer_key
        /enterprise/api/enterprise-customer-invite-key/{enterprise_customer_key}/link-user

        Given a enterprise_customer_key, link user to the appropriate enterprise.

        If the key is not found, returns 404
        If the key is not valid, returns 422
        If we create an `EnterpriseCustomerUser` returns 201
        If an `EnterpriseCustomerUser` if found returns 200
        """
        enterprise_customer_key = get_object_or_404(
            models.EnterpriseCustomerInviteKey,
            uuid=pk
        )

        if not enterprise_customer_key.is_valid:
            return Response(
                {"detail": "Enterprise customer invite key is not valid"},
                status=status.HTTP_422_UNPROCESSABLE_ENTITY,
            )

        enterprise_customer = enterprise_customer_key.enterprise_customer

        enterprise_user, created = models.EnterpriseCustomerUser.all_objects.get_or_create(
            user_id=request.user.id,
            enterprise_customer=enterprise_customer,
        )

        response_body = {
            "enterprise_customer_slug": enterprise_customer.slug,
            "enterprise_customer_uuid": enterprise_customer.uuid,
        }
        headers = self.get_success_headers(response_body)

        track_enterprise_user_linked(
            request.user.id,
            pk,
            enterprise_customer.uuid,
            created,
        )

        if created:
            enterprise_user.invite_key = enterprise_customer_key
            enterprise_user.save()
            return Response(response_body, status=HTTP_201_CREATED, headers=headers)

        elif not enterprise_user.active or not enterprise_user.linked:
            try:
                models.EnterpriseCustomerUser.all_objects.link_user(
                    enterprise_customer,
                    request.user.email
                )
            except LinkUserToEnterpriseError:
                return Response(status=status.HTTP_422_UNPROCESSABLE_ENTITY)

            enterprise_user.refresh_from_db()
            enterprise_user.invite_key = enterprise_customer_key
            enterprise_user.save()

        return Response(response_body, status=HTTP_200_OK, headers=headers)



"""
Class for transmitting content metadata to SuccessFactors.
"""
import json
import logging

from integrated_channels.integrated_channel.transmitters.content_metadata import ContentMetadataTransmitter
from integrated_channels.sap_success_factors.client import SAPSuccessFactorsAPIClient
from integrated_channels.utils import log_exception

LOGGER = logging.getLogger(__name__)


class SapSuccessFactorsContentMetadataTransmitter(ContentMetadataTransmitter):
    """
    This transmitter transmits exported content metadata to SAPSF.
    """

    def __init__(self, enterprise_configuration, client=SAPSuccessFactorsAPIClient):
        """
        Use the ``SAPSuccessFactorsAPIClient`` for content metadata transmission to SAPSF.
        """
        super().__init__(
            enterprise_configuration=enterprise_configuration,
            client=client
        )

    def _filter_api_response(self, response, content_id):
        """
        Filter the response from SAPSF to only include the content
        based on the content_id
        """
        try:
            parsed_response = json.loads(response)
            parsed_response["ocnCourses"] = [item for item in parsed_response["ocnCourses"]
                                             if item["courseID"] == content_id]
            filtered_response = json.dumps(parsed_response)
            return filtered_response
        except Exception as exc:  # pylint: disable=broad-except
            log_exception(
                self.enterprise_configuration,
                f'Failed to filter the API response: '
                f'{exc}',
                content_id
            )
            return response

    def transmit(self, create_payload, update_payload, delete_payload):
        """
        Transmit method overriding base transmissions. Due to rate limiting on SAP
        that prevents us from calling the post endpoint multiple times in one run,
        we prioritize the transmissions in DELETE, UPDATE, CREATE order
        """

        delete_payload_results, create_payload_results, update_payload_results = {}, {}, {}

        if delete_payload:
            self._log_info_for_each_item_map(delete_payload, 'transmitting delete')
            delete_payload_results = self._transmit_delete(delete_payload)
        elif update_payload:
            self._log_info_for_each_item_map(update_payload, 'transmitting update')
            update_payload_results = self._transmit_update(update_payload)
        elif create_payload:
            self._log_info_for_each_item_map(create_payload, 'transmitting create')
            create_payload_results = self._transmit_create(create_payload)
        return create_payload_results, update_payload_results, delete_payload_results

    def _prepare_items_for_transmission(self, channel_metadata_items):
        return {
            'ocnCourses': channel_metadata_items
        }



# Generated by Django 3.2.8 on 2021-12-20 17:24

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0151_add_is_active_to_invite_key'),
        ('moodle', '0007_auto_20210923_1727'),
    ]

    operations = [
        migrations.AlterField(
            model_name='moodleenterprisecustomerconfiguration',
            name='enterprise_customer',
            field=models.ForeignKey(help_text='Enterprise Customer associated with the configuration.', on_delete=django.db.models.deletion.CASCADE, to='enterprise.enterprisecustomer'),
        ),
    ]



# Generated by Django 3.2.12 on 2022-04-05 20:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('integrated_channel', '0011_contentmetadataitemtransmission_plugin_configuration_id'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='contentmetadataitemtransmission',
            unique_together=set(),
        ),
    ]



# Generated by Django 3.2.12 on 2022-07-22 12:54

from django.db import migrations, models


def move_progress_v2_to_v3(apps, schema_editor):
    """
    Moves all the 'progress' and 'progress_v2' data types in config to 'progress_v3'
    """
    ReportingConfig = apps.get_model('enterprise', 'EnterpriseCustomerReportingConfiguration')
    ReportingConfig.objects.filter(data_type__in=['progress_v2', 'progress']).update(data_type='progress_v3')

class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0160_add_enable_portal_learner_credit_management_screen'),
    ]

    operations = [
        migrations.RunPython(move_progress_v2_to_v3, migrations.RunPython.noop),
        migrations.AlterField(
            model_name='enterprisecustomerreportingconfiguration',
            name='data_type',
            field=models.CharField(choices=[('progress_v3', 'progress_v3'), ('catalog', 'catalog'), ('engagement', 'engagement'), ('grade', 'grade'), ('completion', 'completion'), ('course_structure', 'course_structure')], default='progress_v3', help_text='The type of data this report should contain.', max_length=20, verbose_name='Data Type'),
        ),
    ]



# Generated by Django 3.2.23 on 2024-01-15 07:54

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('integrated_channel', '0030_integratedchannelapirequestlogs'),
        ('cornerstone', '0030_auto_20231010_1654'),
    ]

    operations = [
        migrations.CreateModel(
            name='CornerstoneAPIRequestLogs',
            fields=[
                ('integratedchannelapirequestlogs_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='integrated_channel.integratedchannelapirequestlogs')),
                ('user_agent', models.CharField(max_length=255)),
                ('user_ip', models.GenericIPAddressField(blank=True, null=True)),
            ],
            bases=('integrated_channel.integratedchannelapirequestlogs',),
        ),
    ]



"""
Class for transmitting learner data to SuccessFactors.
"""

import logging

from enterprise.models import EnterpriseCustomerUser
from integrated_channels.integrated_channel.transmitters.learner_data import LearnerTransmitter
from integrated_channels.sap_success_factors.client import SAPSuccessFactorsAPIClient
from integrated_channels.utils import generate_formatted_log

LOGGER = logging.getLogger(__name__)


class SapSuccessFactorsLearnerTransmitter(LearnerTransmitter):
    """
    This endpoint is intended to receive learner data routed from the integrated_channel app that is ready to be
    sent to SuccessFactors.
    """

    INCLUDE_GRADE_FOR_COMPLETION_AUDIT_CHECK = False

    def __init__(self, enterprise_configuration, client=SAPSuccessFactorsAPIClient):
        """
        By default, use the ``SAPSuccessFactorsAPIClient`` for learner data transmission to SAPSF.
        """
        super().__init__(
            enterprise_configuration=enterprise_configuration,
            client=client
        )

    def transmit(self, payload, **kwargs):
        """
        Send a completion status call to SAP SuccessFactors using the client.

        Args:
            payload: The learner data exporter for SAP SuccessFactors
        """
        kwargs['app_label'] = 'sap_success_factors'
        kwargs['model_name'] = 'SapSuccessFactorsLearnerDataTransmissionAudit'
        kwargs['remote_user_id'] = 'sapsf_user_id'
        super().transmit(payload, **kwargs)

    def handle_transmission_error(self, learner_data, client_exception):
        """Handle the case where the employee on SAPSF's side is marked as inactive."""
        try:
            sys_msg = str(client_exception.message)
            ecu = EnterpriseCustomerUser.objects.get(
                enterprise_enrollments__id=learner_data.enterprise_course_enrollment_id)

        except AttributeError:
            pass
        else:
            if sys_msg and 'user account is inactive' in sys_msg:
                ecu.active = False
                ecu.save()
                LOGGER.warning(
                    generate_formatted_log(
                        self.enterprise_configuration.channel_code(),
                        ecu.enterprise_customer.uuid,
                        ecu.user_id,
                        None,
                        f'User with LMS ID {ecu.user_id}, ECU ID {ecu.id} is a '
                        f'former employee of {ecu.enterprise_customer} '
                        'and has been marked inactive in SAPSF. Now marking inactive internally.'
                    )
                )



"""
Transmits consenting enterprise learner data to the integrated channels.
"""
from logging import getLogger

from django.apps import apps
from django.contrib import auth
from django.core.management.base import BaseCommand
from django.db import transaction
from django.db.models import Max
from django.utils.translation import gettext as _

User = auth.get_user_model()
LOGGER = getLogger(__name__)


class Command(BaseCommand):
    """
    Management command which removes the duplicated transmission audit records for integration channels
    """
    help = _('''
    Transmit Enterprise learner course completion data for the given EnterpriseCustomer.
    ''')

    def handle(self, *args, **options):
        """
        Remove the duplicated transmission audit records for integration channels.
        """
        # Multiple transmission records were being saved against single enterprise_course_enrollment_id in case
        # transmission fails against course and course run id. Job of this management command is to keep the latest
        # record for enterprise_course_enrollment_id that doesn't start with "course-v1: and delete all other records."
        channel_learner_audit_models = [
            ('moodle', 'MoodleLearnerDataTransmissionAudit'),
            ('blackboard', 'BlackboardLearnerDataTransmissionAudit'),
            ('canvas', 'CanvasLearnerDataTransmissionAudit'),
            ('degreed2', 'Degreed2LearnerDataTransmissionAudit'),
            ('sap_success_factors', 'SapSuccessFactorsLearnerDataTransmissionAudit'),
        ]
        for app_label, model_name in channel_learner_audit_models:
            model_class = apps.get_model(app_label=app_label, model_name=model_name)

            latest_records_without_prefix = (
                model_class.objects.exclude(course_id__startswith='course-v1:')
                .values('enterprise_course_enrollment_id').annotate(most_recent_transmission_id=Max('id'))
            )

            LOGGER.info(
                f'{app_label} channel has {latest_records_without_prefix.count()} records without prefix'
            )

            # Delete all duplicate records for each enterprise_course_enrollment_id
            with transaction.atomic():
                for entry in latest_records_without_prefix:
                    enterprise_course_enrollment_id = entry['enterprise_course_enrollment_id']
                    most_recent_transmission_id = entry['most_recent_transmission_id']

                    # Delete all records except the latest one without "course-v1:"
                    duplicate_records_to_delete = (
                        model_class.objects
                        .filter(enterprise_course_enrollment_id=enterprise_course_enrollment_id)
                        .exclude(id=most_recent_transmission_id)
                    )
                    LOGGER.info(
                        f'{app_label} channel - {duplicate_records_to_delete.count()} duplicate records are deleted '
                        f' for enrollment id {enterprise_course_enrollment_id}'
                    )
                    duplicate_records_to_delete.delete()



"""
Python API for interacting with content metadata.
"""
import logging

from edx_django_utils.cache import TieredCache
from requests.exceptions import HTTPError

from django.conf import settings

from enterprise.api_client.enterprise_catalog import EnterpriseCatalogApiClient
from enterprise.cache_utils import versioned_cache_key

logger = logging.getLogger(__name__)

DEFAULT_CACHE_TIMEOUT = getattr(settings, 'CONTENT_METADATA_CACHE_TIMEOUT', 60 * 5)


def get_and_cache_content_metadata(content_key, timeout=None, coerce_to_parent_course=False):
    """
    Returns the metadata corresponding to the requested ``content_key``
    REGARDLESS of catalog/customer associations.

    The response is cached in a ``TieredCache`` (meaning in both the RequestCache,
    _and_ the django cache for the configured expiration period).

    Returns: A dict with content metadata for the given key.
    Raises: An HTTPError if there's a problem getting the content metadata
      via the enterprise-catalog service.
    """
    cache_key = versioned_cache_key('get_content_metadata_content_identifier', content_key, coerce_to_parent_course)
    cached_response = TieredCache.get_cached_response(cache_key)
    if cached_response.is_found:
        logger.info(f'cache hit for content_key {content_key} and coerce_to_parent_course={coerce_to_parent_course}')
        return cached_response.value

    try:
        result = EnterpriseCatalogApiClient().get_content_metadata_content_identifier(
            content_id=content_key,
            coerce_to_parent_course=coerce_to_parent_course,
        )
    except HTTPError as exc:
        raise exc

    if not result:
        logger.warning('No content found for content_key %s', content_key)
        return {}

    logger.info(
        'Fetched catalog for content_key %s and coerce_to_parent_course=%s. Result = %s',
        content_key,
        coerce_to_parent_course,
        result,
    )
    TieredCache.set_all_tiers(cache_key, result, timeout or DEFAULT_CACHE_TIMEOUT)
    return result


def get_and_cache_customer_content_metadata(enterprise_customer_uuid, content_key, timeout=None):
    """
    Returns the metadata corresponding to the requested
    ``content_key`` within catalogs associated to the provided ``enterprise_customer``.

    The response is cached in a ``TieredCache`` (meaning in both the RequestCache,
    _and_ the django cache for the configured expiration period).

    Returns: A dict with content metadata for the given key.
    Raises: An HTTPError if there's a problem getting the content metadata
      via the enterprise-catalog service.
    """
    cache_key = versioned_cache_key(
        'get_customer_content_metadata_content_identifier',
        enterprise_customer_uuid,
        content_key,
    )
    cached_response = TieredCache.get_cached_response(cache_key)
    if cached_response.is_found:
        logger.info(f'cache hit for enterprise customer {enterprise_customer_uuid} and content {content_key}')
        return cached_response.value

    try:
        result = EnterpriseCatalogApiClient().get_customer_content_metadata_content_identifier(
            enterprise_uuid=enterprise_customer_uuid,
            content_id=content_key,
        )
    except HTTPError as exc:
        raise exc

    if not result:
        logger.warning(
            'No content found for customer %s and content_key %s',
            enterprise_customer_uuid,
            content_key,
        )
        return {}

    logger.info(
        'Fetched catalog for customer %s and content_key %s. Result = %s',
        enterprise_customer_uuid,
        content_key,
        result,
    )
    TieredCache.set_all_tiers(cache_key, result, timeout or DEFAULT_CACHE_TIMEOUT)
    return result


def get_and_cache_enterprise_contains_content_items(enterprise_customer_uuid, content_keys, timeout=None):
    """
    Returns whether the provided content keys are present in the catalogs
    associated with the provided enterprise customer, in addition to a list
    of catalog UUIDs containing the content keys.

    The response is cached in a ``TieredCache``.

    Returns: Dict containing `contains_content_items` and `catalog_list` properties.
    Raises: An HTTPError if there's a problem checking catalog inclusion
      via the enterprise-catalog service.
    """
    cache_key = versioned_cache_key('get_enterprise_contains_content_items', enterprise_customer_uuid, content_keys)
    cached_response = TieredCache.get_cached_response(cache_key)
    if cached_response.is_found:
        logger.info(f'cache hit for enterprise customer {enterprise_customer_uuid} and content keys {content_keys}')
        return cached_response.value

    try:
        result = EnterpriseCatalogApiClient().enterprise_contains_content_items(
            enterprise_uuid=enterprise_customer_uuid,
            content_ids=content_keys,
        )
    except HTTPError as exc:
        raise exc

    if not result:
        logger.warning('No content items found for customer %s', enterprise_customer_uuid)
        return {}

    logger.info(
        'Fetched content catalog inclusion for enterprise customer %s and content keys %s. Result = %s',
        enterprise_customer_uuid, content_keys, result,
    )
    TieredCache.set_all_tiers(cache_key, result, timeout or DEFAULT_CACHE_TIMEOUT)
    return result



# Generated by Django 4.2.10 on 2024-03-01 17:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0201_auto_20240227_2227'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisegroup',
            name='applies_to_all_contexts',
            field=models.BooleanField(default=False, help_text='When enabled, all learners connected to the org will be considered a member.', verbose_name='Set group membership to the entire org of learners.'),
        ),
        migrations.AddField(
            model_name='historicalenterprisegroup',
            name='applies_to_all_contexts',
            field=models.BooleanField(default=False, help_text='When enabled, all learners connected to the org will be considered a member.', verbose_name='Set group membership to the entire org of learners.'),
        ),
    ]



# Generated by Django 4.2.16 on 2025-02-19 20:20

from django.db import migrations, connection


class Migration(migrations.Migration):
    dependencies = [
        ('enterprise', '0229_enterprisecustomeruser_user_fk_and_more'),
    ]

    db_engine = connection.settings_dict['ENGINE']
    if 'sqlite3' in db_engine:
        operations = []
    else:
        operations = [
            # Custom SQL to add indexes with zero-downtime options
            migrations.RunSQL(
                sql="""
                    ALTER TABLE enterprise_enterprisecustomeruser
                    ADD INDEX idx_enterprise_user_fk (user_fk),
                    ALGORITHM=INPLACE,
                    LOCK=NONE;

                    ALTER TABLE enterprise_historicalenterprisecustomeruser
                    ADD INDEX idx_historical_enterprise_user_fk (user_fk),
                    ALGORITHM=INPLACE,
                    LOCK=NONE;
                """,
                reverse_sql="""
                    ALTER TABLE enterprise_enterprisecustomeruser DROP INDEX idx_enterprise_user_fk;
                    ALTER TABLE enterprise_historicalenterprisecustomeruser DROP INDEX idx_historical_enterprise_user_fk;
                """
            ),
        ]



"""
Waffle toggles for enterprise features within the LMS.
"""

from edx_toggles.toggles import WaffleFlag

ENTERPRISE_NAMESPACE = 'enterprise'
ENTERPRISE_LOG_PREFIX = 'Enterprise: '

# .. toggle_name: enterprise.TOP_DOWN_ASSIGNMENT_REAL_TIME_LCM
# .. toggle_implementation: WaffleFlag
# .. toggle_default: False
# .. toggle_description: Enables top-down assignment
# .. toggle_use_cases: open_edx
# .. toggle_creation_date: 2023-09-15
TOP_DOWN_ASSIGNMENT_REAL_TIME_LCM = WaffleFlag(
    f'{ENTERPRISE_NAMESPACE}.top_down_assignment_real_time_lcm',
    __name__,
    ENTERPRISE_LOG_PREFIX,
)

# .. toggle_name: enterprise.feature_prequery_search_suggestions
# .. toggle_implementation: WaffleFlag
# .. toggle_default: False
# .. toggle_description: Enables prequery search suggestions
# .. toggle_use_cases: open_edx
# .. toggle_creation_date: 2024-01-31
FEATURE_PREQUERY_SEARCH_SUGGESTIONS = WaffleFlag(
    f'{ENTERPRISE_NAMESPACE}.feature_prequery_search_suggestions',
    __name__,
    ENTERPRISE_LOG_PREFIX,
)

# .. toggle_name: enterprise.enterprise_groups_v1
# .. toggle_implementation: WaffleFlag
# .. toggle_default: False
# .. toggle_description: Enables enterprise groups feature
# .. toggle_use_cases: open_edx
# .. toggle_creation_date: 2024-02-29
ENTERPRISE_GROUPS_V1 = WaffleFlag(
    f'{ENTERPRISE_NAMESPACE}.enterprise_groups_v1',
    __name__,
    ENTERPRISE_LOG_PREFIX,
)

# .. toggle_name: enterprise.enterprise_customer_support_tool
# .. toggle_implementation: WaffleFlag
# .. toggle_default: False
# .. toggle_description: Enables the enterprise customer support tool
# .. toggle_use_cases: open_edx
# .. toggle_creation_date: 2024-07-17
ENTERPRISE_CUSTOMER_SUPPORT_TOOL = WaffleFlag(
    f'{ENTERPRISE_NAMESPACE}.enterprise_customer_support_tool',
    __name__,
    ENTERPRISE_LOG_PREFIX,
)

# .. toggle_name: enterprise.enterprise_groups_v2
# .. toggle_implementation: WaffleFlag
# .. toggle_default: False
# .. toggle_description: Enables enterprise groups feature
# .. toggle_use_cases: open_edx
# .. toggle_creation_date: 2024-09-24
ENTERPRISE_GROUPS_V2 = WaffleFlag(
    f'{ENTERPRISE_NAMESPACE}.enterprise_groups_v2',
    __name__,
    ENTERPRISE_LOG_PREFIX,
)

# .. toggle_name: enterprise.enterprise_learner_bff_enabled
# .. toggle_implementation: WaffleFlag
# .. toggle_default: False
# .. toggle_description: Enables the enterprise learner BFF
# .. toggle_use_cases: open_edx
# .. toggle_creation_date: 2024-12-16
ENTERPRISE_LEARNER_BFF_ENABLED = WaffleFlag(
    f'{ENTERPRISE_NAMESPACE}.learner_bff_enabled',
    __name__,
    ENTERPRISE_LOG_PREFIX,
)

# .. toggle_name: enterprise.admin_portal_learner_profile_view_enabled
# .. toggle_implementation: WaffleFlag
# .. toggle_default: False
# .. toggle_description: Enables an admin to view a learner's profile
# .. toggle_use_cases: open_edx
# .. toggle_creation_date: 2025-02-12
ADMIN_PORTAL_LEARNER_PROFILE_VIEW_ENABLED = WaffleFlag(
    f'{ENTERPRISE_NAMESPACE}.admin_portal_learner_profile_view_enabled',
    __name__,
    ENTERPRISE_LOG_PREFIX,
)

# .. toggle_name: enterprise.catalog_query_search_filters_enabled
# .. toggle_implementation: WaffleFlag
# .. toggle_default: False
# .. toggle_description: Enables filtering search results by catalog queries vs. enterprise-specific attributes.
# .. toggle_use_cases: open_edx
# .. toggle_creation_date: 2025-03-02
CATALOG_QUERY_SEARCH_FILTERS_ENABLED = WaffleFlag(
    f'{ENTERPRISE_NAMESPACE}.catalog_query_search_filters_enabled',
    __name__,
    ENTERPRISE_LOG_PREFIX,
)


def top_down_assignment_real_time_lcm():
    """
    Returns whether top-down assignment and real time LCM feature flag is enabled.
    """
    return TOP_DOWN_ASSIGNMENT_REAL_TIME_LCM.is_enabled()


def feature_prequery_search_suggestions():
    """
    Returns whether the prequery search suggestion feature flag is enabled.
    """
    return FEATURE_PREQUERY_SEARCH_SUGGESTIONS.is_enabled()


def enterprise_groups_v1():
    """
    Returns whether the enterprise groups feature flag is enabled.
    """
    return ENTERPRISE_GROUPS_V1.is_enabled()


def enterprise_groups_v2():
    """
    Returns whether the enterprise groups v2 feature flag is enabled.
    """
    return ENTERPRISE_GROUPS_V2.is_enabled()


def enterprise_customer_support_tool():
    """
    Returns whether the enterprise customer support tool is enabled.
    """
    return ENTERPRISE_CUSTOMER_SUPPORT_TOOL.is_enabled()


def enterprise_learner_bff_enabled():
    """
    Returns whether the enterprise learner BFF is enabled.
    """
    return ENTERPRISE_LEARNER_BFF_ENABLED.is_enabled()


def admin_portal_learner_profile_view_enabled():
    """
    Returns whether the learner profile view in admin portal is enabled.
    """
    return ADMIN_PORTAL_LEARNER_PROFILE_VIEW_ENABLED.is_enabled()


def catalog_query_search_filters_enabled():
    """
    Returns whether the catalog query search filters feature flag is enabled.
    """
    return CATALOG_QUERY_SEARCH_FILTERS_ENABLED.is_enabled()


def enterprise_features():
    """
    Returns a dict of enterprise Waffle-based feature flags.
    """
    return {
        'top_down_assignment_real_time_lcm': top_down_assignment_real_time_lcm(),
        'feature_prequery_search_suggestions': feature_prequery_search_suggestions(),
        'enterprise_groups_v1': enterprise_groups_v1(),
        'enterprise_customer_support_tool': enterprise_customer_support_tool(),
        'enterprise_groups_v2': enterprise_groups_v2(),
        'enterprise_learner_bff_enabled': enterprise_learner_bff_enabled(),
        'admin_portal_learner_profile_view_enabled': admin_portal_learner_profile_view_enabled(),
        'catalog_query_search_filters_enabled': catalog_query_search_filters_enabled(),
    }



"""
URL definitions for edX Enterprise's Consent API endpoint.
"""

from django.urls import include, path

urlpatterns = [
    path('v1/', include('consent.api.v1.urls'),
         name='consent_api_v1'
         ),
]



# Generated by Django 3.2.21 on 2023-10-02 09:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0188_alter_learnercreditenterprisecourseenrollment_enterprise_course_enrollment_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomer',
            name='enable_pathways',
            field=models.BooleanField(default=True, help_text='If checked, the learners will be able to see the pathways on the learner portal dashboard.', verbose_name='Display pathways screen'),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomer',
            name='enable_pathways',
            field=models.BooleanField(default=True, help_text='If checked, the learners will be able to see the pathways on the learner portal dashboard.', verbose_name='Display pathways screen'),
        ),
    ]



"""
    Serializer for Degreed configuration.
"""
from rest_framework import serializers

from integrated_channels.api.serializers import EnterpriseCustomerPluginConfigSerializer
from integrated_channels.degreed.models import DegreedEnterpriseCustomerConfiguration


class DegreedConfigSerializer(EnterpriseCustomerPluginConfigSerializer):
    class Meta:
        model = DegreedEnterpriseCustomerConfiguration
        extra_fields = (
            'key',
            'secret',
            'degreed_company_id',
            'degreed_base_url',
            'degreed_user_id',
            'degreed_user_password',
            'provider_id',
        )
        fields = EnterpriseCustomerPluginConfigSerializer.Meta.fields + extra_fields



# Generated by Django 2.2.24 on 2021-09-08 10:59

import collections
from django.db import migrations
import enterprise.constants
import enterprise.validators
import jsonfield.encoder
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0142_auto_20210907_2059'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisecustomercatalog',
            name='content_filter',
            field=jsonfield.fields.JSONField(blank=True, default={}, dump_kwargs={'cls': jsonfield.encoder.JSONEncoder, 'ensure_ascii': False, 'indent': 4, 'separators': (',', ':')}, help_text="Query parameters which will be used to filter the discovery service's search/all endpoint results, specified as a Json object. An empty Json object means that all available content items will be included in the catalog.", load_kwargs={'object_pairs_hook': collections.OrderedDict}, null=True, validators=[enterprise.validators.validate_content_filter_fields]),
        ),
        migrations.AlterField(
            model_name='enterprisecustomercatalog',
            name='enabled_course_modes',
            field=jsonfield.fields.JSONField(default=enterprise.constants.json_serialized_course_modes, help_text='Ordered list of enrollment modes which can be displayed to learners for course runs in this catalog.'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomercatalog',
            name='content_filter',
            field=jsonfield.fields.JSONField(blank=True, default={}, dump_kwargs={'cls': jsonfield.encoder.JSONEncoder, 'ensure_ascii': False, 'indent': 4, 'separators': (',', ':')}, help_text="Query parameters which will be used to filter the discovery service's search/all endpoint results, specified as a Json object. An empty Json object means that all available content items will be included in the catalog.", load_kwargs={'object_pairs_hook': collections.OrderedDict}, null=True, validators=[enterprise.validators.validate_content_filter_fields]),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomercatalog',
            name='enabled_course_modes',
            field=jsonfield.fields.JSONField(default=enterprise.constants.json_serialized_course_modes, help_text='Ordered list of enrollment modes which can be displayed to learners for course runs in this catalog.'),
        ),
    ]



"""
URL definitions for enterprise_learner_portal API endpoint.
"""

from django.urls import include, path

urlpatterns = [
    path(
        'v1/',
        include('enterprise_learner_portal.api.v1.urls'),
        name='v1'
    )
]



# Generated by Django 3.2.10 on 2022-02-01 16:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0151_add_is_active_to_invite_key'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomeruser',
            name='should_inactivate_other_customers',
            field=models.BooleanField(default=True, help_text='When enabled along with `active`, all other linked enterprise customers for this user will be marked as inactive upon save.'),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomeruser',
            name='should_inactivate_other_customers',
            field=models.BooleanField(default=True, help_text='When enabled along with `active`, all other linked enterprise customers for this user will be marked as inactive upon save.'),
        ),
    ]



# Generated by Django 2.2.20 on 2021-06-08 19:21

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0131_auto_20210517_0924'),
    ]

    operations = [
        migrations.AddField(
            model_name='enrollmentnotificationemailtemplate',
            name='template_type',
            field=models.CharField(choices=[('SELF_ENROLL', 'Self Enrollment Template'), ('ADMIN_ENROLL', 'Admin Enrollment Template')], default='SELF_ENROLL', help_text='Use either SELF_ENROLL or ADMIN_ENROLL', max_length=255),
        ),
        migrations.AddField(
            model_name='historicalenrollmentnotificationemailtemplate',
            name='template_type',
            field=models.CharField(choices=[('SELF_ENROLL', 'Self Enrollment Template'), ('ADMIN_ENROLL', 'Admin Enrollment Template')], default='SELF_ENROLL', help_text='Use either SELF_ENROLL or ADMIN_ENROLL', max_length=255),
        ),
        migrations.AlterField(
            model_name='enrollmentnotificationemailtemplate',
            name='enterprise_customer',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='enterprise_enrollment_template', to='enterprise.EnterpriseCustomer'),
        ),
    ]
